@@ -4,18 +4,19 @@ use strict;
 use warnings;
 use 5.006;
 use warnings::register;
-use AnyEvent;
 use IPC::Open3 qw( open3 );
 use Scalar::Util qw( reftype );
 use Symbol qw( gensym );
 use AnyEvent::Open3::Simple::Process;
 use Carp qw( croak );
 use File::Temp ();
+use constant _is_native_win32 => $^O eq 'MSWin32';
+use constant _detect => _is_native_win32() ? 'idle' : 'child';
 
 # ABSTRACT: Interface to open3 under AnyEvent
-our $VERSION = '0.85'; # VERSION
-
+our $VERSION = '0.86'; # VERSION
 
+ 
 sub new
 {
   my $default_handler = sub { };
@@ -27,8 +28,10 @@ sub new
   $self{$_} = $args->{$_} || $default_handler for qw( on_stdout on_stderr on_start on_exit on_signal on_fail on_error on_success );
   $self{impl} = $args->{implementation} 
              || $ENV{ANYEVENT_OPEN3_SIMPLE}
-             || ($^O eq 'MSWin32' ? 'idle' : 'child');
-  croak "unknown implementation $self{impl}" unless $self{impl} =~ /^(idle|child)$/;
+             || _detect();
+  croak "unknown implementation $self{impl}" unless $self{impl} =~ /^(idle|child|mojo)$/;
+  $self{impl} = _detect() 
+    if $self{impl} eq 'mojo' && do { require Mojo::Reactor; Mojo::Reactor->detect eq 'Mojo::Reactor::EV' };
   bless \%self, $class;
 }
 
@@ -62,6 +65,19 @@ sub run
     open TEMP, '<&=', $file;
     $child_stdin = '<&TEMP';
   }
+
+  if($self->{impl} =~ /^(child|idle)$/)
+  {
+    require AnyEvent;
+    AnyEvent::detect();
+    require AnyEvent::Open3::Simple::Idle if $self->{impl} eq 'idle';
+  }
+  elsif($self->{impl} eq 'mojo')
+  {
+    require Mojo::Reactor;
+    require Mojo::IOLoop;
+    require AnyEvent::Open3::Simple::Mojo;
+  }
   
   my $pid = eval { open3 $child_stdin, $child_stdout, $child_stderr, $program, @arguments };
   
@@ -87,12 +103,6 @@ sub run
     ref($ref) eq 'ARRAY' ? push @$ref, $input : $ref->($proc, $input);
   };
 
-  $watcher_stdout = AnyEvent->io(
-    fh   => $child_stdout,
-    poll => 'r',
-    cb   => $stdout_callback,
-  ) unless $^O eq 'MSWin32';
-  
   my $stderr_callback = sub {
     my $input = <$child_stderr>;
     return unless defined $input;
@@ -101,11 +111,20 @@ sub run
     ref($ref) eq 'ARRAY' ? push @$ref, $input : $ref->($proc, $input);
   };
 
-  $watcher_stderr = AnyEvent->io(
-    fh   => $child_stderr,
-    poll => 'r',
-    cb   => $stderr_callback,
-  ) unless $^O eq 'MSWin32';
+  if(!_is_native_win32() && $self->{impl} =~ /^(idle|child)$/)
+  {
+    $watcher_stdout = AnyEvent->io(
+      fh   => $child_stdout,
+      poll => 'r',
+      cb   => $stdout_callback,
+    ) unless _is_native_win32();
+  
+    $watcher_stderr = AnyEvent->io(
+      fh   => $child_stderr,
+      poll => 'r',
+      cb   => $stderr_callback,
+    ) unless _is_native_win32();
+  }
 
   my $watcher_child;
 
@@ -156,22 +175,55 @@ sub run
     undef $proc;
   };
 
-  if($self->{impl} eq 'idle')
+  if($self->{impl} eq 'mojo')
   {
+    my($selout, $selerr);
+    
+    if(_is_native_win32())
+    {
+      require IO::Select;
+      $selout = IO::Select->new($child_stdout);
+      $selerr = IO::Select->new($child_stderr);
+    }
+
+    my $reactor = Mojo::IOLoop->singleton->reactor;
+    my $id;
+    $id = Mojo::IOLoop->recurring(0.25 => sub {
+      AnyEvent::Open3::Simple::Mojo::_watcher($pid, sub {
+        $end_cb->(@_);
+        Mojo::IOLoop->remove($id);
+        if(_is_native_win32())
+        {
+          $stdout_callback->() if $selout->can_read(0);
+          $stderr_callback->() if $selerr->can_read(0);
+        }
+        else
+        {
+          $reactor->remove($child_stdout);
+          $reactor->remove($child_stderr);
+        }
+      });
+    });
+   
+  }
+  elsif($self->{impl} eq 'idle')
+  {
+    my($selout, $selerr);
+    
+    if(_is_native_win32())
+    {
+      require IO::Select;
+      $selout = IO::Select->new($child_stdout);
+      $selerr = IO::Select->new($child_stderr);
+    }
+
     $watcher_child = AnyEvent->idle(cb => sub {
-      my $kid = eval q{
-        use POSIX ":sys_wait_h";
-        waitpid($pid, WNOHANG);
-      };
-      
-      if($^O eq 'MSWin32')
+      if(_is_native_win32())
       {
-        $stdout_callback->() if !eof $child_stdout;
-        $stderr_callback->() if !eof $child_stderr;
+        $stdout_callback->() if $selout->can_read(0);
+        $stderr_callback->() if $selerr->can_read(0);
       }
-      
-      warn "IMPORTANT waitpid failed: $@" if $@;
-      $end_cb->($kid, $?) if $kid == $pid;
+      AnyEvent::Open3::Simple::Idle::_watcher($pid, $end_cb);
     });
   }
   else
@@ -199,7 +251,7 @@ AnyEvent::Open3::Simple - Interface to open3 under AnyEvent
 
 =head1 VERSION
 
-version 0.85
+version 0.86
 
 =head1 SYNOPSIS
 
@@ -271,7 +323,7 @@ Event callbacks have an C<on_> prefix, attributes do not.
 =item * implementation
 
 The implementation to use for detecting process termination.  This should
-be one of C<child> or C<idle>.  On all platforms except for Microsoft
+be one of C<child>, C<idle> or C<mojo>.  On all platforms except for Microsoft
 Windows (but not Cygwin) the default is C<child>.
 
 You can change the default by setting the C<ANYEVENT_OPEN3_SIMPLE>
@@ -279,15 +331,9 @@ environment variable, like this:
 
  % export ANYEVENT_OPEN3_SIMPLE=idle
 
-=over 4
-
-=item L<AnyEvent::Open3::Simple::Process#print>
-
-=item L<AnyEvent::Open3::Simple::Process#say>
-
-=item L<AnyEvent::Open3::Simple#run>
-
-=back
+The C<mojo> implementation is experimental and allows you to use
+L<AnyEvent::Open3::Simple> with L<Mojolicious> but without L<EV>
+(which is usually required for L<AnyEvent>, L<Mojolicious> interaction).
 
 =back
 
